@@ -18,6 +18,9 @@ import re
 from datetime import date, datetime, time, timedelta, timezone
 from typing import Union
 
+from . import errors
+from .utils import change_exception
+
 date_re = re.compile(r'(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})$')
 
 time_re = re.compile(
@@ -82,7 +85,7 @@ def from_unix_seconds(seconds: int) -> datetime:
 
 def parse_date(value: Union[date, StrIntFloat]) -> date:
     """
-    Parse a string and return a datetime.date.
+    Parse a date/int/float/string and return a datetime.date.
 
     Raise ValueError if the input is well formatted but not a valid date.
     Raise ValueError if the input isn't well formatted.
@@ -91,20 +94,22 @@ def parse_date(value: Union[date, StrIntFloat]) -> date:
         return value
 
     number = get_numeric(value)
-    if number:
+    if number is not None:
         return from_unix_seconds(number).date()
 
     match = date_re.match(value)
     if not match:
-        raise ValueError('Invalid date format')
+        raise errors.DateError()
 
     kw = {k: int(v) for k, v in match.groupdict().items()}
-    return date(**kw)
+
+    with change_exception(errors.DateError, ValueError):
+        return date(**kw)
 
 
-def parse_time(value: Union[time, StrIntFloat]) -> time:
+def parse_time(value: Union[time, str]) -> time:
     """
-    Parse a string and return a datetime.time.
+    Parse a time/string and return a datetime.time.
 
     This function doesn't support time zone offsets.
 
@@ -116,18 +121,21 @@ def parse_time(value: Union[time, StrIntFloat]) -> time:
 
     match = time_re.match(value)
     if not match:
-        raise ValueError('Invalid time format')
+        raise errors.TimeError()
 
     kw = match.groupdict()
     if kw['microsecond']:
         kw['microsecond'] = kw['microsecond'].ljust(6, '0')
+
     kw = {k: int(v) for k, v in kw.items() if v is not None}
-    return time(**kw)
+
+    with change_exception(errors.TimeError, ValueError):
+        return time(**kw)
 
 
 def parse_datetime(value: Union[datetime, StrIntFloat]) -> datetime:
     """
-    Parse a string and return a datetime.datetime.
+    Parse a datetime/int/float/string and return a datetime.datetime.
 
     This function supports time zone offsets. When the input contains one,
     the output uses a timezone with a fixed offset from UTC.
@@ -139,16 +147,17 @@ def parse_datetime(value: Union[datetime, StrIntFloat]) -> datetime:
         return value
 
     number = get_numeric(value)
-    if number:
+    if number is not None:
         return from_unix_seconds(number)
 
     match = datetime_re.match(value)
     if not match:
-        raise ValueError('Invalid datetime format')
+        raise errors.DateTimeError()
 
     kw = match.groupdict()
     if kw['microsecond']:
         kw['microsecond'] = kw['microsecond'].ljust(6, '0')
+
     tzinfo = kw.pop('tzinfo')
     if tzinfo == 'Z':
         tzinfo = timezone.utc
@@ -158,14 +167,17 @@ def parse_datetime(value: Union[datetime, StrIntFloat]) -> datetime:
         if tzinfo[0] == '-':
             offset = -offset
         tzinfo = timezone(timedelta(minutes=offset))
+
     kw = {k: int(v) for k, v in kw.items() if v is not None}
     kw['tzinfo'] = tzinfo
-    return datetime(**kw)
+
+    with change_exception(errors.DateTimeError, ValueError):
+        return datetime(**kw)
 
 
 def parse_duration(value: StrIntFloat) -> timedelta:
     """
-    Parse a duration string and return a datetime.timedelta.
+    Parse a duration int/float/string and return a datetime.timedelta.
 
     The preferred format for durations in Django is '%d %H:%M:%S.%f'.
 
@@ -177,13 +189,16 @@ def parse_duration(value: StrIntFloat) -> timedelta:
 
     match = standard_duration_re.match(value) or iso8601_duration_re.match(value)
     if not match:
-        raise ValueError('Invalid duration format')
+        raise errors.DurationError()
 
     kw = match.groupdict()
     sign = -1 if kw.pop('sign', '+') == '-' else 1
     if kw.get('microseconds'):
         kw['microseconds'] = kw['microseconds'].ljust(6, '0')
+
     if kw.get('seconds') and kw.get('microseconds') and kw['seconds'].startswith('-'):
         kw['microseconds'] = '-' + kw['microseconds']
+
     kw = {k: float(v) for k, v in kw.items() if v is not None}
+
     return sign * timedelta(**kw)
